import os
import time
from gettext import gettext as _
import logging
import subprocess

class PlymouthManager():
    def __init__(self):
        # type: (str) -> None
        pass

    def message_to(self,message):
        subprocess.call(["/bin/plymouth", "message", "--text", message])

    def progress_to(self,progress):
        tmp = ("--progress=%d"%progress)
        run_cmd = ["/bin/plymouth","system-update",tmp]
        subprocess.call(run_cmd)

    def check_state(self):
        try:
            args = ["/bin/plymouth","--ping"]
            p = subprocess.run(args,timeout=2,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,text=True)
            if p.returncode == 0:
                logging.info("Check: Plymouthd state start to success...")
            else:
                logging.info("Check: Plymouthd Failed to Boot and Restart boot up again...")
                self._start_plymouth_splash()
        except Exception as exc:
            logging.error(exc) 

    def show(self):
        try:
            if os.path.exists("/sbin/plymouthd"):
                if not self._check_device_type():
                    args = ["/sbin/plymouthd", "--mode=boot","--attach-to-session"]
                else:
                    os.environ["DISPLAY"] = ":0"
                    args = ["/sbin/plymouthd", "--mode=shutdown","--attach-to-session"]
                logging.info("Running plymouth:%r",args)
                subprocess.run(args,timeout=3)
            if os.path.exists("/bin/plymouth"):
                subprocess.Popen(["/bin/plymouth", "show-splash","--wait"])
        except Exception as exc:
            logging.error(exc)

    def quit_retain(self):
        if os.path.exists("/bin/plymouth"):
            subprocess.run(["/bin/plymouth","quit","--retain-splash"])

    def _check_device_type(self):
        try:
            with open("/proc/cpuinfo", 'r') as f:
                for line in f.readlines():
                    if "Hardware" in line and ":" in line:
                        cpuinfos = line.split(':')
                        if "PANGU M900" in cpuinfos[1]:
                            logging.info("Get cpuinfo: PANGU M900")
                            return True
        except Exception as exc:
            logging.error(exc)
        return False
    
    def show_countdown(self,success):
        if success:
            msg = _("The system update is successful and your computer will restart in %s s")
        else:
            msg = _("System update is abnormal and your computer will restart in %s s")

        for i in range(5, 0, -1):
            msges = msg%str(i)
            self.message_to(msges)
            time.sleep(1)
            logging.info(msges)

    def clean_all(self):
        self.message_to("")
        self.progress_to(0)
