#!/usr/bin/python3
from configparser import NoOptionError
from configparser import ConfigParser as SafeConfigParser
import os.path
import os
import shutil
import logging

class UpgradeConfig(SafeConfigParser):
    # 备份还原之前保存的配置
    UPDATE_NOT_RESOTRE_DIR = "/var/lib/kylin-system-updater/remain"

    def __init__(self, datadir="/var/lib/kylin-system-updater/", name="system-updater.conf",defaults_conf=None):
        SafeConfigParser.__init__(self)
        self.datadir = datadir
        self.maincfg = os.path.join(datadir, name)

        self.config_files = []
        # 增加的默认配置，先读取默认配置加载配置项，最终写入到主配置内
        if defaults_conf:
            self.config_files.append(os.path.join(datadir, defaults_conf))
        self.config_files.append(self.maincfg)
        
        # 检查是否存在备份的配置，存在时直接移动到默认位置
        self.remail_conf = os.path.join(self.UPDATE_NOT_RESOTRE_DIR, name)
        if os.path.exists(self.remail_conf):
            shutil.move(self.remail_conf,self.maincfg)

        # 检查配置文件是否存在
        for conf in self.config_files:
            if not os.path.exists(conf):
                logging.warning("Check: Conf(%s) not exits...",conf)
                # self.config_files.remove(conf)

        self.read(self.config_files, encoding='iso-8859-1')
        logging.info("Initialize Upgrade ConfigFile(%s) to success",str(self.config_files))

    def reRead(self):
        self.read(self.config_files, encoding='iso-8859-1')

    def setValue(self, section, option, value=None,is_write = True):
        logging.info("SetValue Section:%s Option:%s Value:%s",section, option, value)
        try:
            self.reRead()
            self.set(section, option,value)
        except Exception as e:
                logging.error("Error: setValue section:%s option:%s value:%s",section, option, value)
                logging.error(str(e))
                return False
        if is_write == True:
            with open(self.maincfg, 'w+') as configfile:
                self.write(configfile)
        return True

    def getWithDefault(self, section, option, default,re_read=False):
        try:
            if re_read == True:
                self.reRead()

            if type(default) == bool:
                return self.getboolean(section, option)
            elif type(default) == float:
                return self.getfloat(section, option)
            elif type(default) == int:
                return self.getint(section, option)
            return self.get(section, option)
        except Exception as e:
            logging.warning(str(e))
            return default

    def getlist(self, section, option):
        try:
            tmp = self.get(section, option)
        except Exception as e:
            logging.error(str(e))
            return []
        items = [x.strip() for x in tmp.split(" ")]
        if '' in items and len(items):
            return []
        return items

    def setListValue(self, section, option, value=None,is_write = True):
        tmp = str(value).replace('[', '').replace(']', '')
        tmp = tmp.replace("'", '').replace(',', '')
        try:
            self.set(section, option,tmp)
        except Exception as e:
            logging.error("setListValue section:%s option:%s",section, option)
            logging.error(str(e))
            return
        if is_write == True:
            with open(self.maincfg, 'w+') as configfile:
                self.write(configfile)

    def getListFromFile(self, section, option):
        try:
            filename = self.get(section, option)
        except NoOptionError:
            return []
        p = os.path.join(self.datadir, filename)
        if not os.path.exists(p):
            logging.error("getListFromFile: no '%s' found" % p)
        with open(p) as f:
            items = [x.strip() for x in f]
        return [s for s in items if not s.startswith("#") and not s == ""]

    # 还原之前保留配置
    def remain_conf(self):
        shutil.copy(self.maincfg, self.remail_conf)

    def optionxform(self, optionstr: str) -> str:
        return optionstr
if __name__ == "__main__":
    pass