import os
import dbus
import time
import fcntl
import shutil
import logging
from gi.repository import Gio, GLib
from gi.repository import GObject
from gettext import gettext as _
from SystemUpdater.Core.system import run_system
from SystemUpdater.Core.PlymouthManager import PlymouthManager

FILELOCK_PATH = "/tmp/lock/"
SHUTDOWN_BLOCK_FILELOCK = "kylin-update.lock"

LOGIND_SLEEP_CONF = "/etc/systemd/sleep.conf.d/"
LOGIND_SLEEP_POSITION = "/etc/systemd/sleep.conf.d/inhibit-sleep.conf"
SLEEP_CONF_POSITION = "/var/lib/kylin-system-updater/inhibit-sleep.conf"
NEED_ROLLBACK_FALG = "/var/lib/kylin-system-updater/need-rollback"

UPDATE_NOT_RESOTRE_DIR = "/var/lib/kylin-system-updater/remain"
EXCEPTION_SHUTDOWN_FLAG = "/var/lib/kylin-system-updater/remain/install-exception-flag"


def unlock_force_sleep():
    try:
        if os.path.exists(LOGIND_SLEEP_POSITION):
            os.remove(LOGIND_SLEEP_POSITION)
            # logging.info("File(%s) is exists and will be delete and unlock sleep",LOGIND_SLEEP_POSITION)
    except Exception as e:
        logging.error(e)


class InhibitShutdownLock():
    # 禁止关机锁文件路径
    def __init__(self):
        self.inhibit_lock = None
        self.pidfile = None

    #安装时禁止关机 进行加锁
    def lock(self, caller='kylin-system-updater',inhibit_sleep = False):
        """
        Send a dbus signal to logind to not suspend the system, it will be
        released when the return value drops out of scope
        """
        self._make_execption_check_flag()

        if inhibit_sleep == True:
            self._lock_sleep()

        if self.inhibit_lock == None:
            try:
                from gi.repository import Gio, GLib
                logging.info("Start acquiring Shutdown lock(block)...")
                connection = Gio.bus_get_sync(Gio.BusType.SYSTEM)
                
                var, fdlist = connection.call_with_unix_fd_list_sync(
                    'org.freedesktop.login1', '/org/freedesktop/login1',
                    'org.freedesktop.login1.Manager', 'Inhibit',
                    GLib.Variant('(ssss)',
                                ('shutdown:sleep',
                                caller, _("Installing or removing software packages."),
                                'block')),
                    None, 0, -1, None, None)
                self.inhibit_lock = Gio.UnixInputStream(fd=fdlist.steal_fds()[var[0]])
                self.LockedPreventShutdown()
                
                logging.info("Shutdown Has been locked...")
                return True
            except Exception as e:
                logging.error(e)
                return False
        else:
            logging.info("No need to acquired lock and Shutdown Has been locked...")
            return False

    #解锁禁止关机 
    def unlock(self):
        try:
            self._remove_check_flag()
            self.unLockedEnableShutdown()
            unlock_force_sleep()
            if self.inhibit_lock != None:
                self.inhibit_lock.close()
                self.inhibit_lock = None
                logging.info("Shutdown Has been unlocked...")
            else:
                logging.info("Not locked and Quitting ...")
        except Exception as e:
            self.inhibit_lock = None
            logging.error("unlock failed." + str(e))

    def _lock_sleep(self):
        try:
            if not os.path.exists(LOGIND_SLEEP_CONF):
                logging.info("File(%s) is not exists and will be create",LOGIND_SLEEP_CONF)
                os.makedirs(LOGIND_SLEEP_CONF)
            shutil.copy(SLEEP_CONF_POSITION, LOGIND_SLEEP_CONF)
            logging.info("File(%s) will be create for forcely inhibit sleep...",LOGIND_SLEEP_CONF)
        except Exception as e:
            logging.error(e)
    
    @staticmethod
    def _check_unlock_sleep():
        try:
            LOGIND_SLEEP_POSITION = "/etc/systemd/sleep.conf.d/inhibit-sleep.conf"
            NEED_ROLLBACK_FALG = "/var/lib/kylin-system-updater/need-rollback"

            if os.path.exists(LOGIND_SLEEP_POSITION):
                os.remove(LOGIND_SLEEP_POSITION)
                logging.warning("File(%s) is exists and will be delete and unlock sleep",LOGIND_SLEEP_POSITION)

            if os.path.exists(NEED_ROLLBACK_FALG):
                os.remove(NEED_ROLLBACK_FALG)
                logging.warning("File(%s) is exists and will be delete and unlock restore",NEED_ROLLBACK_FALG)
        except Exception as e:
            logging.error(e)

    def mark_restore(self):
        with open("/var/lib/kylin-system-updater/need-rollback", 'w') as f:
            pass

    def remove_restore_flag(self):
        if os.path.exists(NEED_ROLLBACK_FALG):
            os.remove(NEED_ROLLBACK_FALG)

    #安装时禁止关机 进行加锁
    def LockedPreventShutdown(self):
        #不为空是表示以及被锁
        if self.pidfile != None:
            logging.error("self.pidfile file disc not is None,Has been locked...")
            return False

        if not os.path.exists(FILELOCK_PATH):
            #不存在创建
            logging.info("File(%s) is not exists and will be create",FILELOCK_PATH)
            os.makedirs(FILELOCK_PATH)
        else:
            #当目录存在时进行删除 不删除进行创建文件的话会报错 
            # file cannot be locked.[Errno 11] Resource temporarily unavailable
            # 资源被占用报错
            shutil.rmtree(FILELOCK_PATH)
            logging.info("File(%s) is exists and will be delete and create",FILELOCK_PATH)
            os.makedirs(FILELOCK_PATH)        

        try:
            self.pidfile = open(os.path.join(FILELOCK_PATH, SHUTDOWN_BLOCK_FILELOCK), "w+")
            fcntl.flock(self.pidfile, fcntl.LOCK_EX | fcntl.LOCK_NB)
            return True
        except Exception as e:
            logging.error("file cannot be locked." + str(e))
            self.pidfile = None
            return False

    #解锁禁止关机 
    def unLockedEnableShutdown(self):
        #未加锁退出
        if not self.pidfile:
            logging.info("Not locked and Quitting ...")
            return False
        try:
            fcntl.flock(self.pidfile, fcntl.LOCK_UN)
            self.pidfile.close()
            self.pidfile = None

            # Fix 修复权限问题 当普通用户无法使用 所以直接删除目录
            if os.path.exists(FILELOCK_PATH):
                shutil.rmtree(FILELOCK_PATH)
                logging.info('Emptying the lockPath(%s) is complete...',FILELOCK_PATH)
            else:
                logging.info("Emptying the lockPath(%s) is Failed...",FILELOCK_PATH)

            return True
        except Exception as e:
            logging.error("unlock failed." + str(e))
            self.pidfile = None
            return False

    def _make_execption_check_flag(self):
        if not os.path.exists(UPDATE_NOT_RESOTRE_DIR):
            os.makedirs(UPDATE_NOT_RESOTRE_DIR)

        with open(EXCEPTION_SHUTDOWN_FLAG, 'w') as file:
            pass  # 不写入任何内容，只创建文件
    
    def _remove_check_flag(self):
        try:
            os.remove(EXCEPTION_SHUTDOWN_FLAG)
        except Exception as e:
            pass
    
    # 存在异常时 检查到时 获取完状态直接删除
    def is_abnormal(self):
        if os.path.exists(EXCEPTION_SHUTDOWN_FLAG):
            self._remove_check_flag()
            return True
        else:
            return False

class CatchShutdownAction(GObject.GObject):
    # 禁止关机锁文件路径
    __gsignals__ = {"trigger_install": (GObject.SIGNAL_RUN_FIRST,
                                GObject.TYPE_NONE,
                                (   
                                    GObject.TYPE_STRING,
                                )),
                    }
    
    def __init__(self):
        GObject.GObject.__init__(self)
        self.inhibit_lock = None
        self.pidfile = None
        self.bus = dbus.SystemBus()
        self.has_trigger = False
        self.signal_handle = None
        self.plymouth = PlymouthManager()

    #安装时禁止关机 进行加锁
    def lock(self, caller='kylin-system-updater',mode='delay'):
        """
        Send a dbus signal to logind to not suspend the system, it will be
        released when the return value drops out of scope
        """
        if self.inhibit_lock == None:
            try:
                from gi.repository import Gio, GLib
                logging.info("Start acquiring Shutdown lock(%s)...",mode)
                connection = Gio.bus_get_sync(Gio.BusType.SYSTEM)
                
                var, fdlist = connection.call_with_unix_fd_list_sync(
                    'org.freedesktop.login1', '/org/freedesktop/login1',
                    'org.freedesktop.login1.Manager', 'Inhibit',
                    GLib.Variant('(ssss)',
                                ('shutdown:sleep',
                                caller, _("Installing or removing software packages."),
                                mode)),
                    None, 0, -1, None, None)
                self.inhibit_lock = Gio.UnixInputStream(fd=fdlist.steal_fds()[var[0]])
                logging.info("Shutdown Has been locked...")
                return True
            except Exception as e:
                logging.error(e)
                return False
        else:
            logging.info("No need to acquired lock and Shutdown Has been locked...")
            return False

    #解锁禁止关机 
    def unlock(self):
        try:
            if self.inhibit_lock != None:
                self.inhibit_lock.close()
                self.inhibit_lock = None
                logging.info("Shutdown Has been unlocked...")
            else:
                logging.info("Not locked and Quitting ...")
            if self.signal_handle:
                logging.info("remove signal handle...")
                self.signal_handle.remove()
                self.signal_handle = None
        except Exception as e:
            self.inhibit_lock = None
            logging.error("unlock failed." + str(e))

    def connect_logind(self):
        try:
            logind_proxy = self.bus.get_object('org.freedesktop.login1', '/org/freedesktop/login1')
            def prepare_for_shutdown_handler(active):
                """ Handle PrepareForShutdown() """
                if not active:
                    logging.warning("PrepareForShutdown(false) received, "
                                    "this should not happen")

                logging.info("Prepare For Shutdown arrived, starting final iterations....")
                # 当再次收到关机信号时 仅强制禁止关闭
                if self.has_trigger:
                    for i in range(3):
                        self._stop_logind_service()
                        time.sleep(1)
                    self._switch_to_tty1()
                    return 

                device_status = self._check_device_status()

                self._pre_install_preparation()

                self.has_trigger = True
                self.emit("trigger_install",device_status)

            self.signal_handle = logind_proxy.connect_to_signal(
                    "PrepareForShutdown", prepare_for_shutdown_handler)

            self._start_monitor_device()
        except Exception as exc:
            logging.error(exc)

    def _pre_install_preparation(self):
        self.plymouth.show()
        mesa = _("The system is installing updates. Please do not turn off your computer. ")
        self.plymouth.message_to(mesa)
        self.plymouth.progress_to(5)

        self._stop_lightdm_service()
        self._switch_to_tty1()
        for i in range(3):
            self._stop_logind_service()
            time.sleep(1)
        self.unlock()
        self._switch_to_tty1()
        self._mask_lightdm_service()

    def _stop_logind_service(self):
        try:
            with open("/usr/bin/touchpad-state", 'w') as file:
                file.write("# fix bug:202730 ,The scripts was cleared by system upgrade.\n")
            run_system(["systemctl", "stop","systemd-logind.service"])
        except Exception as exc:
            logging.error(exc)

    def _stop_lightdm_service(self):
        run_system(["systemctl","stop","lightdm.service"])

    def _mask_lightdm_service(self):
        run_system(["systemctl","mask","lightdm.service"])
    
    @staticmethod
    def _unmask_lightdm_service():
        SERVICE_FLAG = "lightdm.service"
        mask_lightdm = ["systemctl","unmask",SERVICE_FLAG]
        file_path = os.path.join("/etc/systemd/system",SERVICE_FLAG)

        if os.path.exists(file_path):
            logging.info("unmask lightdm service")
            run_system(mask_lightdm)

    def _switch_to_tty1(self):
        run_system(["/usr/bin/chvt","1"])

    def _check_device_status(self):
        try:
            state = "reboot"
            with open("/tmp/monitor-dbus-upgrade.log", 'r') as f:
                text = f.read()
                if "member=PowerOff" in text:
                    state = "shutdown"
            return state
        except Exception as exc:
            logging.error(exc)
            return "reboot"

    def _start_monitor_device(self): 
        try:
            args = ["/usr/share/kylin-system-updater/scripts/monitor-reboot"]
            run_system(args)
        except Exception as exc:
            logging.error(exc)


class BaseShutdownLock(GObject.GObject):
    # 禁止关机锁文件路径
    __gsignals__ = {"trigger_install": (GObject.SIGNAL_RUN_FIRST,
                                GObject.TYPE_NONE,
                                (   
                                    GObject.TYPE_STRING,
                                )),
                    }
    
    def __init__(self):
        GObject.GObject.__init__(self)
        self.pidfile = None


    def lock(self):
        pass

    def unlock(self):
        pass

    def _inhibit_shutdown(self,caller='kylin-system-updater',mode="block"):
        inhibit_lock = None
        try:
            logging.info("Start acquiring Shutdown lock(%s)...",mode)
            connection = Gio.bus_get_sync(Gio.BusType.SYSTEM)
            
            var, fdlist = connection.call_with_unix_fd_list_sync(
                'org.freedesktop.login1', '/org/freedesktop/login1',
                'org.freedesktop.login1.Manager', 'Inhibit',
                GLib.Variant('(ssss)',
                            ('shutdown:sleep',
                            caller, _("Installing or removing software packages."),
                            'block')),
                None, 0, -1, None, None)
            inhibit_lock = Gio.UnixInputStream(fd=fdlist.steal_fds()[var[0]])
            logging.info("Shutdown Has been locked...")
        except Exception as e:
            logging.warning(e)

        return inhibit_lock